#version 130
precision mediump float;


uniform vec3 cameraLocation;

uniform  vec4 ambientLight;

uniform  vec3 mainLightLocation;
uniform  vec4 mainLightColor;
//uniform float mainLigthRadius;

in vec3 transNormal;
in vec3 viewDirection;
in vec3 lightDirection;


// Shader Mode
uniform int shaderMode;

//Material Colors
uniform vec4 Ka;
uniform vec4 Kd;
uniform vec4 Ke;
uniform vec4 Ks;
uniform float Ns;
uniform float d;

//Color Bias
uniform vec4 colorBias;

// texture coordinates and the texture image data
in vec2 texture_coordinates; 
uniform sampler2D textureData;
uniform sampler2D fboAlphaData;


vec4 totalLight;

float alphaValue;



// Motion Blur Vars
in vec2 velocity;



void main(){
	
	vec3  nLightDirection = normalize( lightDirection );
	vec3  nNormal         = normalize( transNormal );
	float NDotL           = dot( nNormal, nLightDirection ); 
	
	vec3  nReflection     = normalize( ( ( 2.0 * nNormal ) * NDotL ) - lightDirection ); 
	vec3  nViewDirection  = normalize( viewDirection );
	float RDotV           = max( 0.0, dot( nReflection, nViewDirection ) );
	
	vec4  baseColor;
	
	
	
	
	// SIMPLE
	if(shaderMode == 0) {
	
		gl_FragColor = colorBias * Kd * ( ambientLight + (mainLightColor * dot(transNormal, normalize(mainLightLocation))));
		gl_FragColor.a = d;
	
	
	}
	
	// TEXTURE
	else if(shaderMode == 1) {
	
		gl_FragColor = colorBias * texture(textureData, texture_coordinates); 
	
	
	}
	
	//TEXTURE_LIGHTING
	else if(shaderMode == 2) {
	
	
		baseColor = texture( textureData, texture_coordinates );
		
		alphaValue = baseColor.a;
					
		vec4  totalAmbient   = baseColor * ambientLight; // * Ka;
		vec4  totalDiffuse   = baseColor * Kd * NDotL; 
		vec4  totalSpecular  = Ks * ( pow( RDotV, 10.0 ) ); //TODO variable power of specularity
	
		gl_FragColor = colorBias * (totalAmbient + totalDiffuse + totalSpecular);
		
		gl_FragColor.a = d * alphaValue * colorBias.a;
	
	
	
	
	}
	
	// MOTION BLUR FBO 
	else if(shaderMode == 3) { 
	
		int numOfSamples = 16;
			
		vec2 deltaVelocity = velocity / numOfSamples;
		
		vec2 currentTexCoord = texture_coordinates;		
		
		// Get the initial color at this pixel
		vec4 color = texture(textureData, currentTexCoord);  
		
		for(int i = 1; i <= numOfSamples; i++) {  

			currentTexCoord += deltaVelocity;

			// Sample the color buffer along the velocity vector.  
			vec4 currentColor = texture(textureData, currentTexCoord);  

			// Add the current color to our color sum.  
			color += currentColor;  

		}  
		
		// Average all of the samples to get the final blur color.  
		vec4 finalColor = color / numOfSamples;  
		
		
		gl_FragColor = finalColor;

		gl_FragColor.a = d;

	
	} 
	
	

}